/* news_mgr.c

   Written by Curtis Olson
   Started April 9, 1993
*/


#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <utime.h>

#include "error_mgr.h"
#include "news_mgr.h"


time_t get_last_news_time() {
    /* Determine the time of the user's last news reading from the modification
       date of the file HOME/DEFAULT_NEWS_TIME */

    struct stat buf;
    char time_file[256];

    /* Get home directory */
    sprintf(time_file, "%s", getenv("HOME"));
    /* printf("Home directory:  '%s'\n", time_file); */

    strcat(time_file, "/");
    strcat(time_file, DEFAULT_NEWS_TIME);

    if (stat(time_file, &buf) == 0) {
	/* found file, so return it's last modification date */
	return buf.st_mtime;
    } else {
	/* Cannot stat HOME/DEFAULT_NEWS_TIME so return a time of 0 */
        return 0;
    }	
}


void set_last_news_time() {
    /* Update the time of the user's last news reading by updating the 
       modification date of the file HOME/DEFAULT_NEWS_TIME */

    char time_file[256];
    char msg1[256], msg2[256];
    FILE *f;

    /* Get home directory */
    sprintf(time_file, "%s", getenv("HOME"));

    strcat(time_file, "/");
    strcat(time_file, DEFAULT_NEWS_TIME);

    if (utime(time_file, NULL) == 0) {
	/* successfully updated file */
    } else {
	/* problem updating file */
	if (errno == ENOENT) {
	    /* file doesn't exist, so lets create it ... */
	    if ( (f = fopen(time_file, "w")) == NULL) {
		/* Can't update or create the file, lets report the problem */
		sprintf(msg1, "cannot update or create '%s', ", time_file);
		sprintf(msg2, "%s date of last news read not changed.", msg1);
		e_msg(msg2);
	    } else {
	        fclose(f);
	    }
	}
    }
}


void display(char file[], char name[], time_t t) {
    /* cat the file to screen */
    FILE *f;
    char msg[256];
    int c;

    if ( (f = fopen(file, "r")) == NULL ) {
	sprintf(msg, "cannot display '%s'.", file);
	e_msg(msg);
    }

    /* Print a header */
    printf("\n------------------------------\n");
    printf("%s - %s", name, ctime(&t));
    printf("------------------------------\n");

    /* Print the text */
    while ( (c = getc(f)) != EOF ) {
	switch (c) {
	    case 0:	putc('^', stdout);
			putc('@', stdout);
			break;
	    case 1:	/* ^A */
	    case 2:	/* ^B */
	    case 3:	/* ^C */
	    case 4:	/* ^D */
	    case 5:	/* ^E */
	    case 6:	/* ^F */
	    case 11:	/* ^K */
	    case 14:	/* ^N */
	    case 15:	/* ^O */
	    case 16:	/* ^P */
	    case 17:	/* ^Q */
	    case 18:	/* ^R */
	    case 19:	/* ^S */
	    case 20:	/* ^T */
	    case 21:	/* ^U */
	    case 22:	/* ^V */
	    case 23:	/* ^W */
	    case 24:	/* ^X */
	    case 25:	/* ^Y */
	    case 26:	/* ^Z */
			putc('^', stdout);
			putc(c + 'A' - 1, stdout);
			break;
	    default:	putc(c, stdout);
			break;
	}
    }

    fclose(f);
}


int process_news(char file[], int text, int all, int names, int numbers, 
	int t) {
    char root_path[256];
    char file_path[256];
    char msg[256];
    DIR *d;
    struct dirent *de;
    struct stat buf;
    int count = 0;

    /* Get news directory */
    /* try user's environement first */
    if ( getenv("NEWS_PATH") == NULL ) {
	/* nothing in the user's environment, so use the default directory */
	/* DEFAULT_NEWS_PATH is defined in news_mgr.h */
	strcpy(root_path, DEFAULT_NEWS_PATH);
    } else {
        sprintf(root_path, "%s", getenv("NEWS_PATH"));
    }

    if ( (d = opendir(root_path)) == NULL ) {
	sprintf(msg, "cannot open '%s'.", root_path);
	e_msg(msg);
    }

    if (!strlen(file)) {
        while ( (de = readdir(d)) != NULL ) {
	    strcpy(file_path, root_path);
	    strcat(file_path, "/");
	    strcat(file_path, de->d_name);
 	    if (stat(file_path, &buf) == 0) {
	        /* printf("%s:\t%d\t%s", file_path, buf.st_mtime, 
		          ctime(&buf.st_mtime));
	        */
	        if ( (all || (buf.st_mtime > t)) 
			&& (strcmp(de->d_name, ".") != 0)
			&& (strcmp(de->d_name, "..") != 0) ) {
		    /* only deal with necessary files */
	            if (text) {
		        /* we wish to diplay file contents */
		        display(file_path, de->d_name, buf.st_mtime);
	            } else {
		        /* we only want names or quantities */
		        if (names) {
		            /* display names */
		            if (count == 0) {
			        /* first file */
			        printf("You have news: ");
		            }
		            printf(" %s", de->d_name);
		        }
	            }
		    count++;
	        }
	    } else {
	        sprintf(msg, "cannot stat '%s', Error %d.", file_path, errno);
	        e_msg(msg);
	    }	
	}
    } else {
	/* display a specific news file */
	strcpy(file_path, root_path);
	strcat(file_path, "/");
	strcat(file_path, file);
 	if (stat(file_path, &buf) == 0) {
	    display(file_path, file, buf.st_mtime);
	} else {
	    sprintf(msg, "cannot stat '%s', Error %d.", file_path, errno);
	    e_msg(msg);
	}	
    }

    closedir(d);

    if ( names && (count > 0) ) {
	/* end with a carriage return */
	printf("\n");
    } else if ( numbers && (count > 0) ) {
	if (count == 1) {
	    printf("You have news:  %d item.\n", count);
	} else {
	    printf("You have news:  %d items.\n", count);
	}
    }

    return count;
}


int news(char file[], int text, int all, int names, int numbers) {
    time_t t;
    int count;

    /* calculate last time news was read */
    if (!all) {
        t = get_last_news_time();
    } else {
	t = 0;
    }
    /* printf("news_time = %d\n", t); */

    /* process the news */
    count = process_news(file, text, all, names, numbers, t);

    /* update the last time news was read */
    if (text && !all) {
	/* only update if we were reading news -- not just checking names */
 	/* also, don't update if the -a option was specified */
        set_last_news_time();
    }

    return count;
}
